@@ -14,7 +14,7 @@ use std::sync::RwLock;
 use std::time::Duration;
 use std::time::Instant;
 use submap::{BroadcastMap, SubMap};
-use tokio::io::{AsyncReadExt, AsyncWriteExt, BufReader, BufWriter};
+use tokio::io::{AsyncReadExt, AsyncWriteExt, BufReader};
 use tokio::net::{TcpListener, TcpStream, UnixListener, UnixStream};
 #[cfg(feature = "rpc")]
 use tokio::sync::Mutex;
@@ -23,6 +23,7 @@ use tokio::time;
 
 use crate::{Error, ErrorKind, GREETINGS, PROTOCOL_VERSION};
 
+use crate::comm::{Flush, TtlBufWriter};
 use crate::ERR_DATA;
 use crate::ERR_NOT_SUPPORTED;
 use crate::RESPONSE_OK;
@@ -65,20 +66,19 @@ type BrokerClient = Arc<ElbusClient>;
 
 macro_rules! make_confirm_channel {
     ($qos: expr) => {
-        match $qos {
-            QoS::No => Ok(None),
-            QoS::Processed => {
-                let (tx, rx) = tokio::sync::oneshot::channel();
-                let _r = tx.send(Ok(()));
-                Ok(Some(rx))
-            }
+        if $qos.needs_ack() {
+            let (tx, rx) = tokio::sync::oneshot::channel();
+            let _r = tx.send(Ok(()));
+            Ok(Some(rx))
+        } else {
+            Ok(None)
         }
     };
 }
 
 macro_rules! send {
     ($db:expr, $client:expr, $target:expr, $header: expr,
-     $buf:expr, $payload_pos:expr, $len: expr) => {{
+     $buf:expr, $payload_pos:expr, $len: expr, $realtime: expr) => {{
         $client.r_frames.fetch_add(1, atomic::Ordering::SeqCst);
         $client.r_bytes.fetch_add($len, atomic::Ordering::SeqCst);
         $db.r_frames.fetch_add(1, atomic::Ordering::SeqCst);
@@ -101,6 +101,7 @@ macro_rules! send {
                 header: $header,
                 buf: $buf,
                 payload_pos: $payload_pos,
+                realtime: $realtime,
             });
             tx.send(frame).await.map_err(Into::into)
         } else {
@@ -111,7 +112,7 @@ macro_rules! send {
 
 macro_rules! send_broadcast {
     ($db:expr, $client:expr, $target:expr, $header: expr,
-     $buf:expr, $payload_pos:expr, $len: expr) => {{
+     $buf:expr, $payload_pos:expr, $len: expr, $realtime: expr) => {{
         $client.r_frames.fetch_add(1, atomic::Ordering::SeqCst);
         $client.r_bytes.fetch_add($len, atomic::Ordering::SeqCst);
         $db.r_frames.fetch_add(1, atomic::Ordering::SeqCst);
@@ -127,6 +128,7 @@ macro_rules! send_broadcast {
                 header: $header,
                 buf: $buf,
                 payload_pos: $payload_pos,
+                realtime: $realtime,
             });
             $db.w_frames
                 .fetch_add(subs.len() as u64, atomic::Ordering::SeqCst);
@@ -143,7 +145,7 @@ macro_rules! send_broadcast {
 
 macro_rules! publish {
     ($db:expr, $client:expr, $topic:expr, $header: expr,
-     $buf:expr, $payload_pos:expr, $len: expr) => {{
+     $buf:expr, $payload_pos:expr, $len: expr, $realtime: expr) => {{
         $client.r_frames.fetch_add(1, atomic::Ordering::SeqCst);
         $client.r_bytes.fetch_add($len, atomic::Ordering::SeqCst);
         $db.r_frames.fetch_add(1, atomic::Ordering::SeqCst);
@@ -159,6 +161,7 @@ macro_rules! publish {
                 header: $header,
                 buf: $buf,
                 payload_pos: $payload_pos,
+                realtime: $realtime,
             });
             $db.w_frames
                 .fetch_add(subs.len() as u64, atomic::Ordering::SeqCst);
@@ -246,7 +249,16 @@ impl AsyncClient for Client {
         qos: QoS,
     ) -> Result<OpConfirm, Error> {
         let len = payload.len() as u64;
-        send!(self.db, self.client, target, None, payload.to_vec(), 0, len)?;
+        send!(
+            self.db,
+            self.client,
+            target,
+            None,
+            payload.to_vec(),
+            0,
+            len,
+            qos.is_realtime()
+        )?;
         make_confirm_channel!(qos)
     }
     #[inline]
@@ -265,7 +277,8 @@ impl AsyncClient for Client {
             Some(header.to_vec()),
             payload.to_vec(),
             0,
-            len
+            len,
+            qos.is_realtime()
         )?;
         make_confirm_channel!(qos)
     }
@@ -277,7 +290,16 @@ impl AsyncClient for Client {
         qos: QoS,
     ) -> Result<OpConfirm, Error> {
         let len = payload.len() as u64;
-        send_broadcast!(self.db, self.client, target, None, payload.to_vec(), 0, len);
+        send_broadcast!(
+            self.db,
+            self.client,
+            target,
+            None,
+            payload.to_vec(),
+            0,
+            len,
+            qos.is_realtime()
+        );
         make_confirm_channel!(qos)
     }
     #[inline]
@@ -288,7 +310,16 @@ impl AsyncClient for Client {
         qos: QoS,
     ) -> Result<OpConfirm, Error> {
         let len = payload.len() as u64;
-        publish!(self.db, self.client, topic, None, payload.to_vec(), 0, len);
+        publish!(
+            self.db,
+            self.client,
+            topic,
+            None,
+            payload.to_vec(),
+            0,
+            len,
+            qos.is_realtime()
+        );
         make_confirm_channel!(qos)
     }
     #[inline]
@@ -616,25 +647,25 @@ impl RpcHandlers for BrokerRpcHandlers {
         match event.parse_method()? {
             "test" => {
                 if !params.is_empty() {
-                    return Err(RpcError::params());
+                    return Err(RpcError::params(None));
                 }
                 Ok(Some(RPC_OK.to_vec()))
             }
             "info" => {
                 if !params.is_empty() {
-                    return Err(RpcError::params());
+                    return Err(RpcError::params(None));
                 }
                 Ok(Some(rmp_serde::to_vec_named(&Broker::info())?))
             }
             "stats" => {
                 if !params.is_empty() {
-                    return Err(RpcError::params());
+                    return Err(RpcError::params(None));
                 }
                 Ok(Some(rmp_serde::to_vec_named(&self.db.stats())?))
             }
             "client.list" => {
                 if !params.is_empty() {
-                    return Err(RpcError::params());
+                    return Err(RpcError::params(None));
                 }
                 let db = self.db.clients.read().unwrap();
                 let mut clients: Vec<ClientInfo> = db
@@ -655,7 +686,7 @@ impl RpcHandlers for BrokerRpcHandlers {
                 clients.sort();
                 Ok(Some(rmp_serde::to_vec_named(&ClientList { clients })?))
             }
-            _ => Err(RpcError::method()),
+            _ => Err(RpcError::method(None)),
         }
     }
     async fn handle_notification(&self, _event: RpcEvent) {}
@@ -684,7 +715,8 @@ fn prepare_unix_source(_addr: tokio::net::unix::SocketAddr) -> Option<String> {
 }
 
 macro_rules! spawn_server {
-    ($self: expr, $path: expr, $listener: expr, $buf_size: expr, $timeout: expr, $tp: expr,
+    ($self: expr, $path: expr, $listener: expr, $buf_size: expr,
+     $buf_ttl: expr, $timeout: expr, $tp: expr,
      $prepare: ident, $prepare_source: ident) => {{
         let socket_path = $path.to_owned();
         let db = $self.db.clone();
@@ -704,7 +736,7 @@ macro_rules! spawn_server {
                         }
                         let (reader, writer) = stream.into_split();
                         let reader = BufReader::with_capacity($buf_size, reader);
-                        let writer = BufWriter::with_capacity($buf_size, writer);
+                        let writer = TtlBufWriter::new(writer, $buf_size, $buf_ttl, $timeout);
                         let cdb = db.clone();
                         let name = socket_path.clone();
                         let client_source = $prepare_source(addr);
@@ -737,11 +769,11 @@ macro_rules! spawn_server {
 struct PeerHandlerParams<R, W>
 where
     R: AsyncReadExt + Unpin,
-    W: AsyncWriteExt + Unpin + Send + 'static,
+    W: AsyncWriteExt + Unpin + Send + Sync + 'static,
 {
     db: Arc<BrokerDb>,
     reader: R,
-    writer: W,
+    writer: TtlBufWriter<W>,
     timeout: Duration,
     queue_size: usize,
     tp: ElbusClientType,
@@ -823,6 +855,7 @@ impl Broker {
         &mut self,
         path: &str,
         buf_size: usize,
+        buf_ttl: Duration,
         timeout: Duration,
     ) -> Result<(), Error> {
         let _r = tokio::fs::remove_file(path).await;
@@ -832,6 +865,7 @@ impl Broker {
             path,
             listener,
             buf_size,
+            buf_ttl,
             timeout,
             ElbusClientType::LocalIpc,
             prepare_unix_stream,
@@ -843,6 +877,7 @@ impl Broker {
         &mut self,
         path: &str,
         buf_size: usize,
+        buf_ttl: Duration,
         timeout: Duration,
     ) -> Result<(), Error> {
         let listener = TcpListener::bind(path).await?;
@@ -851,6 +886,7 @@ impl Broker {
             path,
             listener,
             buf_size,
+            buf_ttl,
             timeout,
             ElbusClientType::Tcp,
             prepare_tcp_stream,
@@ -952,6 +988,7 @@ impl Broker {
                     rmp_serde::to_vec_named(&params)
                         .map_err(Error::data)?
                         .into(),
+                    QoS::No,
                 )
                 .await?;
                 Ok(())
@@ -980,7 +1017,7 @@ impl Broker {
     async fn handle_peer<R, W>(params: PeerHandlerParams<R, W>) -> Result<(), Error>
     where
         R: AsyncReadExt + Unpin,
-        W: AsyncWriteExt + Unpin + Send + 'static,
+        W: AsyncWriteExt + Unpin + Send + Sync + 'static,
     {
         let timeout = params.timeout;
         let mut reader = params.reader;
@@ -989,8 +1026,7 @@ impl Broker {
         let db = params.db;
         macro_rules! write_and_flush {
             ($buf: expr) => {
-                time::timeout(timeout, writer.write_all($buf)).await??;
-                time::timeout(timeout, writer.flush()).await??;
+                time::timeout(timeout, writer.write($buf, Flush::Instant)).await??;
             };
         }
         let mut buf = GREETINGS.to_vec();
@@ -1038,26 +1074,8 @@ impl Broker {
         let writer_fut = tokio::spawn(async move {
             while let Ok(frame) = rx.recv().await {
                 macro_rules! write_data {
-                    ($data: expr) => {
-                        if !$data.is_empty() {
-                            match time::timeout(timeout, writer.write_all($data)).await {
-                                Ok(result) => {
-                                    if let Err(e) = result {
-                                        pretty_error!(w_name, Into::<Error>::into(&e));
-                                        break;
-                                    }
-                                }
-                                Err(_) => {
-                                    error!("client {} error: timeout", w_name);
-                                    break;
-                                }
-                            }
-                        }
-                    };
-                }
-                macro_rules! flush {
-                    () => {
-                        match time::timeout(timeout, writer.flush()).await {
+                    ($data: expr, $flush: expr) => {
+                        match time::timeout(timeout, writer.write($data, $flush)).await {
                             Ok(result) => {
                                 if let Err(e) = result {
                                     pretty_error!(w_name, Into::<Error>::into(&e));
@@ -1072,8 +1090,7 @@ impl Broker {
                     };
                 }
                 if frame.kind == FrameKind::Prepared {
-                    write_data!(&frame.buf);
-                    flush!();
+                    write_data!(&frame.buf, frame.realtime.into());
                 } else {
                     let sender = frame.sender.as_ref().unwrap().as_bytes();
                     let topic = frame.topic.as_ref().map(String::as_bytes);
@@ -1089,19 +1106,18 @@ impl Broker {
                     let frame_len = extra_len + frame.buf.len() - frame.payload_pos + 1;
                     #[allow(clippy::cast_possible_truncation)]
                     buf.extend_from_slice(&(frame_len as u32).to_le_bytes()); // bytes 1-4
-                    buf.push(0x00); // byte 5 - reserved
+                    buf.push(if frame.realtime { 1 } else { 0 }); // byte 5 - reserved
                     buf.extend_from_slice(sender);
                     buf.push(0x00);
                     if let Some(t) = topic.as_ref() {
                         buf.extend_from_slice(t);
                         buf.push(0x00);
                     };
-                    write_data!(&buf);
+                    write_data!(&buf, Flush::No);
                     if let Some(header) = frame.header() {
-                        write_data!(header);
+                        write_data!(header, Flush::No);
                     }
-                    write_data!(frame.payload());
-                    flush!();
+                    write_data!(frame.payload(), frame.realtime.into());
                 }
             }
         });
@@ -1139,7 +1155,7 @@ impl Broker {
             let mut buf = vec![0; len as usize];
             time::timeout(timeout, reader.read_exact(&mut buf)).await??;
             macro_rules! send_ack {
-                ($code:expr) => {
+                ($code:expr, $realtime: expr) => {
                     let mut buf = Vec::with_capacity(6);
                     buf.push(OP_ACK);
                     buf.extend_from_slice(op_id);
@@ -1160,6 +1176,7 @@ impl Broker {
                             header: None,
                             buf,
                             payload_pos: 0,
+                            realtime: $realtime,
                         }))
                         .await?;
                 };
@@ -1182,8 +1199,8 @@ impl Broker {
                             trace!("elbus client {} subscribed to topic {}", client, topic);
                         }
                     }
-                    if qos == QoS::Processed {
-                        send_ack!(RESPONSE_OK);
+                    if qos.needs_ack() {
+                        send_ack!(RESPONSE_OK, qos.is_realtime());
                     }
                 }
                 FrameOp::UnsubscribeTopic => {
@@ -1203,8 +1220,8 @@ impl Broker {
                             trace!("elbus client {} unsubscribed from topic {}", client, topic);
                         }
                     }
-                    if qos == QoS::Processed {
-                        send_ack!(RESPONSE_OK);
+                    if qos.needs_ack() {
+                        send_ack!(RESPONSE_OK, qos.is_realtime());
                     }
                 }
                 _ => {
@@ -1217,26 +1234,40 @@ impl Broker {
                     match op {
                         FrameOp::Message => {
                             let len = buf.len() as u64;
-                            if let Err(e) = send!(db, client, target, None, buf, payload_pos, len) {
-                                if qos == QoS::Processed {
-                                    send_ack!(e.kind as u8);
+                            let realtime = qos.is_realtime();
+                            if let Err(e) =
+                                send!(db, client, target, None, buf, payload_pos, len, realtime)
+                            {
+                                if qos.needs_ack() {
+                                    send_ack!(e.kind as u8, realtime);
                                 }
-                            } else if qos == QoS::Processed {
-                                send_ack!(RESPONSE_OK);
+                            } else if qos.needs_ack() {
+                                send_ack!(RESPONSE_OK, realtime);
                             }
                         }
                         FrameOp::Broadcast => {
                             let len = buf.len() as u64;
-                            send_broadcast!(db, client, target, None, buf, payload_pos, len);
-                            if qos == QoS::Processed {
-                                send_ack!(RESPONSE_OK);
+                            let realtime = qos.is_realtime();
+                            send_broadcast!(
+                                db,
+                                client,
+                                target,
+                                None,
+                                buf,
+                                payload_pos,
+                                len,
+                                realtime
+                            );
+                            if qos.needs_ack() {
+                                send_ack!(RESPONSE_OK, realtime);
                             }
                         }
                         FrameOp::PublishTopic => {
                             let len = buf.len() as u64;
-                            publish!(db, client, target, None, buf, payload_pos, len);
-                            if qos == QoS::Processed {
-                                send_ack!(RESPONSE_OK);
+                            let realtime = qos.is_realtime();
+                            publish!(db, client, target, None, buf, payload_pos, len, realtime);
+                            if qos.needs_ack() {
+                                send_ack!(RESPONSE_OK, realtime);
                             }
                         }
                         _ => {}
